#
# Copyright (c) 2018 Make.org
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

require 'spec_helper'
require 'parslet'

require_relative '../libraries/intrpl_parser.rb'

# rubocop:disable Lint/InterpolationCheck
intrpls = {
  '#{test}' => { 'intrpl' => '#{test}' },
  '#{\'test\'}' => { 'intrpl' => [{ 'sq' => '\'test\'' }] },
  '#{"test"}' => { 'intrpl' => [{ 'dq' => '"test"' }] },
  '#{"\"test\""}' => { 'intrpl' => [{ 'dq' => '"\"test\""' }] },
  '#{\\\"\"test\""}' => { 'intrpl' => [{ 'dq' => '"\"test\""' }] },
  '#{"\\\\\"\""}' => { 'intrpl' => [{ 'dq' => '"\\\\\"\\""' }] },
  '#{h = {}}' => { 'intrpl' => [{ 'hash' => '{}' }] },
  '#{h = "}"}' => { 'intrpl' => [{ 'dq' => '"}"' }] },
  '#{h = { {"test" => "{}"} }}' => {
    'intrpl' => [{
      'hash' => [{ 'hash' => [{ 'dq' => '"test"' }, { 'dq' => '"{}"' }] }]
    }]
  }
}

not_intrpls = [
  'test',
  '{test}',
  '#{test',
  '#{test}a',
  '#{test"}"'
]
# rubocop:enable Lint/InterpolationCheck

describe IntrplParser do
  parser = IntrplParser.new
  intrpls.each_pair do |test, result|
    it "expression #{test} should be parsed as an interpolation" do
      expect(parser.parse(test)).to eq(result)
    end
  end

  not_intrpls.each do |test|
    it "expression #{test} should not be parsed as an interpolation" do
      expect { parser.parse(test) }.to raise_error(Parslet::ParseFailed)
    end
  end
end
